#!/usr/bin/env node
import { Ts2MdOptions } from "./Ts2MdOptions";
/**
 * Generate Typescript documentation and merge into README.md
 *
 * Attempts to validate options, constructs an instance of Ts2Md with those options, and runs the generation method.
 *
 * 1. Function argument is used if provided.
 *
 * 2. Looks for `./ts2md.json`
 *
 * 3. Default options.
 *
 * Default options are:
 *
 * ```json
 * {
 *   "inputFilename": "./src/index.ts",
 *   "outputFilename": "./apiDoc.md",
 *   "firstHeadingLevel": 2,
 *   "noTitle": true,
 *   "outputReplace": true,
 *   "readmeMerge": true
 * }
 * ```
 *
 * 4. Finally examines command line arguments which are treated as overrides of the options determined
 * by steps 1, 2, 3. Command line arguments can be provided as either:
 *
 * ```
 * --inputFilename ../index.ts
 * ```
 *
 * or
 *
 * ```
 * --inputFilename=../index.ts
 * ```
 *
 * @param options Optional options to control markdown generation.
 * @publicbody
 */
export declare function ts2md(options?: Ts2MdOptions): void;
//# sourceMappingURL=ts2md.d.ts.map